package walidacja;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSSerializer;

import common.MojErrorHandler;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import java.io.FileOutputStream;

public class WalidujWaluty3 {
    public static void main(String[] args) {
        String plikXML = "in/waluty_2022.xml";
        String plikWynikowy = "out/wynik_walidacji.xml";
        String plikXSD = "in/waluty.xsd";
        System.out.println("Początek programu");

        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = dbf.newDocumentBuilder();
            Document doc = builder.parse(plikXML);

            SchemaFactory sf = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema = sf.newSchema(new StreamSource(plikXSD));
            Validator validator = schema.newValidator();
            MojErrorHandler errorHandler = new MojErrorHandler();
            validator.setErrorHandler(errorHandler);

            // Można walidować dokument znajdujący się w pamięci.
            // Wersja validate z dwoma parametrami pozwala zapisać wynik walidacji, czyli dokument z uzwględnionymi regułami zapisanymi w schemaie.
            // Np. uzupełnienie domyślnych atrybutów.
            System.out.println("Start walidacji");
            DOMResult domResult = new DOMResult();
            validator.validate(new DOMSource(doc), domResult);
            Node resultDoc = domResult.getNode();

            if(errorHandler.czyBylBlad()) {
                System.out.println("Koniec walidacji. Dokument zawierał błędy");
            } else {
                System.out.println("Koniec walidacji. Dokument prawidłowy");
            }

            /* Zapisujemy zmieniony uzywajac Load and Save: */
            DOMImplementationLS lsImpl = (DOMImplementationLS)builder.getDOMImplementation().getFeature(
                    "LS", "3.0");
            if(lsImpl == null) {
                System.err.println("Nie udalo sie uzyskac obiektu DOMImplementationLS");
                return;
            }
            LSSerializer ser = lsImpl.createLSSerializer();
            LSOutput out = lsImpl.createLSOutput();
            /* ustawiamy plik */
            out.setByteStream(new FileOutputStream(plikWynikowy));

            System.out.println("Poczatek zapisywania.");
            ser.write(resultDoc, out);
            System.out.println("Koniec.");


        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
